DECLARE EXTERNAL FUNCTION CreateParser
	BLOB
    RETURNS INTEGER BY VALUE
  ENTRY_POINT 'SampleUdf_CreateParser'  MODULE_NAME 'SampleUdf'
#
DECLARE EXTERNAL FUNCTION DestroyParser
	INTEGER
	RETURNS INTEGER BY VALUE
  ENTRY_POINT 'SampleUdf_DestroyParser'  MODULE_NAME 'SampleUdf'
#
DECLARE EXTERNAL FUNCTION GetName
	INTEGER
	RETURNS CSTRING(256)
  ENTRY_POINT 'SampleUdf_GetName'  MODULE_NAME 'SampleUdf'
#
DECLARE EXTERNAL FUNCTION GetFullPath
	INTEGER
	RETURNS CSTRING(256)
  ENTRY_POINT 'SampleUdf_GetFullPath'  MODULE_NAME 'SampleUdf'
#
DECLARE EXTERNAL FUNCTION GetCreationTime
	INTEGER
	RETURNS TIMESTAMP FREE_IT
  ENTRY_POINT 'SampleUdf_GetCreationTime'  MODULE_NAME 'SampleUdf'
#
DECLARE EXTERNAL FUNCTION GetAttributes
	INTEGER
	RETURNS INTEGER
  ENTRY_POINT 'SampleUdf_GetAttributes'  MODULE_NAME 'SampleUdf'
#
DECLARE EXTERNAL FUNCTION GetSize
	INTEGER
	RETURNS BIGINT
  ENTRY_POINT 'SampleUdf_GetSize'  MODULE_NAME 'SampleUdf'
#
CREATE TABLE "RowDataTable" (
	"Id" INTEGER NOT NULL PRIMARY KEY
  , "Value" BLOB
  )
#
CREATE TABLE "FSTable"
 (
	"Id" INTEGER NOT NULL PRIMARY KEY
  , "Name" VARCHAR(256)
  , "FullPath" VARCHAR(256)
  , "CreationTime" TIMESTAMP 
  , "Attributes" INTEGER
  , "Size" BIGINT
  )
#
CREATE GENERATOR "RowDataTable_Generator"
#
CREATE GENERATOR "FSTable_Generator"
#
CREATE PROCEDURE AddRawData
(
  "Value" BLOB
)
RETURNS ("newId" INTEGER)
AS
BEGIN
    "newId" = GEN_ID("RowDataTable_Generator", 1);
    INSERT INTO "RowDataTable" ("Id", "Value") VALUES (:"newId", :"Value");
    SUSPEND;
END
#
CREATE PROCEDURE TransferData
RETURNS ("counter" INTEGER)
AS
DECLARE VARIABLE "tmp" INTEGER;
DECLARE VARIABLE "parserPtr" INTEGER;
DECLARE VARIABLE "Value" BLOB;
DECLARE VARIABLE "Name" VARCHAR(256);
DECLARE VARIABLE "FullPath" VARCHAR(256);
DECLARE VARIABLE "CreationTime" TIMESTAMP;
DECLARE VARIABLE "Attributes" INTEGER;
DECLARE VARIABLE "Size" BIGINT;
BEGIN
	"counter" = 0;
	FOR SELECT "Value" FROM "RowDataTable" INTO :"Value" DO	BEGIN
		SELECT CreateParser(:"Value") FROM rdb$database INTO :"parserPtr";			
		IF ("parserPtr" IS NOT NULL) THEN BEGIN
			SELECT GetName(:"parserPtr") FROM rdb$database INTO :"Name";
			SELECT GetFullPath(:"parserPtr") FROM rdb$database INTO :"FullPath";
			SELECT GetCreationTime(:"parserPtr") FROM rdb$database INTO :"CreationTime";
			SELECT GetAttributes(:"parserPtr") FROM rdb$database INTO :"Attributes";
			SELECT GetSize(:"parserPtr") FROM rdb$database INTO :"Size";
			
			"tmp" = GEN_ID("FSTable_Generator", 1);
			INSERT INTO "FSTable" ("Id", "Name", "FullPath", "CreationTime", "Attributes", "Size")
			VALUES (:"tmp", :"Name", :"FullPath", :"CreationTime", :"Attributes", :"Size");
			"counter" = "counter" + 1;

			SELECT DestroyParser(:"parserPtr") FROM rdb$database INTO :"tmp";			
		END
	END
	SUSPEND;
END